/** 
	A program in Qt
	that use OpenGL drawing primitives
	File: cone.cpp

	Fraga Nov 5, 2001
**/
#include <qpainter.h>
#include <math.h>
#include <ctype.h>
#include "cone.h"

Cone::Cone( QWidget *parent, const char *name )
		: QGLWidget( parent, name )
{
	signo = 1;	
	shade = 0;

	wholight = 0;
	scale = 1;
	angle = 0.0;
	// setMouseTracking( TRUE );
	setFocusPolicy( QWidget::StrongFocus );

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );
}


void Cone::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Cone::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

//
// This method puts the initial coordinates each time window is resized
//
void Cone::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if ( w <= h ) 
		glOrtho ( -1.0, 1.0, -1.0/aspect, 1.0/aspect, -1.0, 1.0);
	else
		glOrtho ( -aspect, aspect, -1.0, 1.0, -1.0, 1.0);

	emit changeSize ( );
}

QSizePolicy Cone::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

/** 
	Function that generate a cube with center in (cx,cy,cz)
	with size value for its side
**/
void Cone::getcone ( float cx, float cy, float cz, float high, float radius, 
		float p[][3], float normal[][3], int n ) 
{
	int i;

	p[0][0] = cx; p[0][1] = cy; p[0][2] = cz + high; 

	float incrang = 360.0/n;

	float theta = atan ( radius / high );
	float sintheta = sin ( theta );
	float costheta = cos ( theta );

	for ( i=1; i<=n; i++ ) { 
		float ang = (i-1)*incrang;

		float sinalpha = sin ( ang * M_PI/180.0 );
		float cosalpha = cos ( ang * M_PI/180.0 );

		p[i][0] = cx + cosalpha * radius; 
		p[i][1] = cy + sinalpha * radius; 
		p[i][2] = cz; 

		normal[i][0] = costheta * cosalpha; 
		normal[i][1] = costheta * sinalpha; 
		normal[i][2] = sintheta; 

		ang += incrang/2.0;

		sinalpha = sin ( ang * M_PI/180.0 );
		cosalpha = cos ( ang * M_PI/180.0 );

		normal[n+i][0] = costheta * cosalpha; 
		normal[n+i][1] = costheta * sinalpha; 
		normal[n+i][2] = sintheta; 

	}
}

void Cone::show_cone ( float p[][3], float normals[][3], int n )
{
	int i;

	for (i = 1; i <= n; i++) {
		glBegin(GL_TRIANGLES);
		// glBegin(GL_LINE_LOOP);

		glColor3f( 0.0, 0.0, 0.8 );
		glNormal3fv( &normals[n+i][0]);
		glVertex3fv( &p[0][0]);

		glColor3f( 1.0, 1.0, 1.0 );
		glNormal3fv( &normals[i][0]);
		glVertex3fv( &p[i][0]);


		int index = i%n + 1;
		glColor3f( 1.0, 1.0, 1.0 );
		glNormal3fv( &normals[index][0]);
		glVertex3fv( &p[index][0]);

		glEnd();
	}
}

void Cone::paintGL(void)
{
	static float position0[] = {0.0, 0.0, 1.0, 1.0};
	static float position1[] = {0.0, 0.0,-1.0, 1.0};

	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if ( !shade ) glShadeModel( GL_FLAT );
	else glShadeModel( GL_SMOOTH );

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	// if ( !wholight ) {
	// 	glPushMatrix( );
	// 	/* viewing transformation  */
	// 	glScalef ( scale, scale, scale);	/* modeling transformation */
	// 	glLightfv(GL_LIGHT0, GL_POSITION, position0);
	// 	glLightfv(GL_LIGHT1, GL_POSITION, position1);
	// 	glPopMatrix( );
	// }

	glRotatef (xrot, 1.0, 0.0, 0.0);
	glRotatef (yrot, 0.0, 1.0, 0.0);

	glScalef ( 0.9, 0.9, 0.9);	  /* modeling transformation */ 

	glPushMatrix( );
	glRotatef ( angle, 1.0, 0.0, 0.0);

	show_cone( cyl1, nor1, SIZECYL );

	glPopMatrix( );

	glBegin(GL_LINES);
		// Draw x axis:
		glColor3f( 1.0, 0.0, 0.0 );
		glVertex3f(-0.1, 0.0, 0.0);
		glVertex3f( 1.1, 0.0, 0.0);
		// Draw y axis:
		glVertex3f( 0.0, -0.1, 0.0);
		glVertex3f( 0.0,  1.1, 0.0);
		// Draw z axis:
		glVertex3f( 0.0, 0.0, -0.1);
		glVertex3f( 0.0, 0.0,  1.1);
	glEnd();
	
	swapBuffers();
}

void Cone::initializeGL( ) 
{
	glClearColor (0.8, 0.8, 0.8, 0.0);	// Background to white

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	getcone ( 0.0, 0.0, 0.0, 1.0, 0.6, cyl1, nor1, SIZECYL );

	// InitMaterials( );

	glEnable (GL_DEPTH_TEST);
	// glEnable( GL_LIGHTING );
	// glShadeModel( GL_FLAT );
	glShadeModel( GL_SMOOTH );
	
}

void Cone::InitMaterials(void)
{
	static float ambient[] = {0.1, 0.1, 0.1, 1.0};
	static float diffuse[] = {1.0, 1.0, 1.0, 1.0};

	// static float position0[] = {10.0, 0.0, 40.0, 1.0};
	// static float position1[] = {-10.0, 0.0, -40.0, 1.0};
	// static float position0[] = { 0.0, 0.0, 10.0, 1.0};
	// static float position1[] = { 0.0, 0.0,-10.0, 1.0};
	static float front_mat_shininess[] = {50.0};
	static float front_mat_specular[] = {1.0, 1.0, 1.0, 1.0};
	static float front_mat_diffuse[] = {1.0, 1.0, 1.0, 1.0};

	// static float back_mat_shininess[] = {40.0};
	// static float back_mat_specular[] = {0.5, 0.5, 0.2, 1.0};
	// static float back_mat_diffuse[] = {1.0, 1.0, 0.2, 1.0};

	static float lmodel_ambient[] = {1.0, 1.0, 1.0, 1.0};
	static float lmodel_twoside[] = {GL_FALSE};

	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	// glLightfv(GL_LIGHT0, GL_POSITION, position0);

	glEnable(GL_LIGHT0);

	glLightfv(GL_LIGHT1, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse);
	// glLightfv(GL_LIGHT1, GL_POSITION, position1);
	glEnable(GL_LIGHT1);

	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside);

	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, front_mat_shininess);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_mat_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, front_mat_diffuse);

	// glPolygonStipple (halftone);
}

void Cone::keyPressEvent ( QKeyEvent * e )
{
	switch ( tolower(e->ascii()) ) {
		case '+':
			scale *= 1.2;
			break;
		case '-':
			scale *= 0.8;
			break;
		case 'l':
			wholight = (++wholight)%2;
			break;
	}
	updateGL( );
}

